#!/usr/bin/env ruby
#
# vgenes2tab.rb - convert KEGG/v-GENES into tab delimited data for MySQL
#
#  Usage:
#
#    % vgenes2tab.rb /bio/db/ideas/vgenes/vgenes > vgenes.tab
#
#   Copyright (C) 2001 KATAYAMA Toshiaki <k@bioruby.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#

require 'bio/db/genes'

while entry = gets(GENES::DELIMITER)

  genes = GENES.new(entry)

  if genes.codon_usage
    cu = genes.codon_usage.join(' ')
  else
    cu = nil
  end

  ary = [
    genes.id,
    genes.entry('type'),
    genes.name,
    genes.definition,
    genes.position,
    genes.fetch('dblinks'),
    cu,
    genes.aalen,
    genes.aaseq,
    genes.ntlen,
    genes.ntseq
  ]

  ary.collect! do |field|
    field = '\N' unless field
    field
  end

  puts ary.join("\t")

end

=begin

CREATE DATABASE IF NOT EXISTS db_name;
CREATE TABLE IF NOT EXISTS db_name.vgenes (
	entry		varchar(30)	not NULL,	# Refseq ID + ORF num
	type		varchar(8),			# CDS, tRNA etc.
	name		varchar(255),
	definition	varchar(255),
	position	varchar(255),
	dblinks		varchar(255),
	codon_usage	text,
	aaseq_len	int,
	aaseq		mediumtext,
	ntseq_len	int,
	ntseq		mediumtext
);
LOAD DATA LOCAL INFILE 'vgenes.tab' INTO TABLE db_name.vgenes;

=end
